const OFF   = 0;    // OFF   or 0 - turn the rule off
const WARN  = 1;    // WARN  or 1 - turn the rule on as a warning (does not affect exit code)
const ERROR = 2;    // ERROR or 2 - turn the rule on as an error (exit code will be 1)


module.exports = {
  parserOptions: {
    ecmaVersion: '2022',
    sourceType: 'module'
  },
  rules: {
    // enforce "for" loop update clause moving the counter in the right direction
    // https://eslint.org/docs/rules/for-direction
    'for-direction': ERROR,

    // enforce that a return statement is present in property getters
    // https://eslint.org/docs/rules/getter-return
    'getter-return': [ERROR, {
      allowImplicit: true
    }],

    // disallow using an async function as a Promise executor
    // https://eslint.org/docs/rules/no-async-promise-executor
    'no-async-promise-executor': ERROR,

    // disallow await inside of loops
    // https://eslint.org/docs/rules/no-await-in-loop
    'no-await-in-loop': ERROR,

    // disallow comparisons to negative zero
    // https://eslint.org/docs/rules/no-compare-neg-zero
    'no-compare-neg-zero': ERROR,

    // disallow assignment in conditional expressions
    // https://eslint.org/docs/rules/no-cond-assign
    'no-cond-assign': [ERROR, 'always'],

    // disallow use of console
    // https://eslint.org/docs/rules/no-console
    'no-console': WARN,

    // disallows expressions where the operation doesn't affect the value
    // https://eslint.org/docs/rules/no-constant-binary-expression
    'no-constant-binary-expression': ERROR,

    // disallow use of constant expressions in conditions
    // https://eslint.org/docs/rules/no-constant-condition
    'no-constant-condition': [WARN, {
      checkLoops: true
    }],

    // disallow control characters in regular expressions
    // https://eslint.org/docs/rules/no-control-regex
    'no-control-regex': ERROR,

    // disallow use of debugger
    // https://eslint.org/docs/rules/no-debugger
    'no-debugger': ERROR,

    // disallow duplicate arguments in functions
    // https://eslint.org/docs/rules/no-dupe-args
    'no-dupe-args': ERROR,

    // disallow duplicate conditions in if-else-if chains
    // https://eslint.org/docs/rules/no-dupe-else-if
    'no-dupe-else-if': ERROR,

    // disallow duplicate keys when creating object literals
    // https://eslint.org/docs/rules/no-dupe-keys
    'no-dupe-keys': ERROR,

    // disallow a duplicate case label.
    // https://eslint.org/docs/rules/no-duplicate-case
    'no-duplicate-case': ERROR,

    // disallow empty statements
    // https://eslint.org/docs/rules/no-empty
    'no-empty': [ERROR, {
      allowEmptyCatch: false
    }],

    // disallow the use of empty character classes in regular expressions
    // https://eslint.org/docs/rules/no-empty-character-class
    'no-empty-character-class': ERROR,

    // disallow assigning to the exception in a catch block
    // https://eslint.org/docs/rules/no-ex-assign
    'no-ex-assign': ERROR,

    // disallow double-negation boolean casts in a boolean context
    // https://eslint.org/docs/rules/no-extra-boolean-cast
    'no-extra-boolean-cast': [ERROR, {
      enforceForLogicalOperands: true
    }],

    // disallow unnecessary parentheses
    // https://eslint.org/docs/rules/no-extra-parens
    'no-extra-parens': [ERROR, 'all', {
      conditionalAssign: true,
      enforceForArrowConditionals: false,
      enforceForSequenceExpressions: false,
      enforceForNewInMemberExpressions: false,
      enforceForFunctionPrototypeMethods: false,
      nestedBinaryExpressions: false,
      returnAssign: false,
      ignoreJSX: 'all' // delegate to eslint-plugin-react
    }],

    // disallow unnecessary semicolons
    // https://eslint.org/docs/rules/no-extra-semi
    'no-extra-semi': ERROR,

    // disallow overwriting functions written as function declarations
    // https://eslint.org/docs/rules/no-func-assign
    'no-func-assign': ERROR,

    // disallow assignments, increments, and decrements of imported bindings
    // https://eslint.org/docs/rules/no-import-assign
    'no-import-assign': ERROR,

    // disallow function or variable declarations in nested blocks
    // https://eslint.org/docs/rules/no-inner-declarations
    'no-inner-declarations': [ERROR, 'both'],

    // disallow invalid regular expression strings in the RegExp constructor
    // https://eslint.org/docs/rules/no-invalid-regexp
    'no-invalid-regexp': ERROR,

    // disallow irregular whitespace outside of strings and comments
    // https://eslint.org/docs/rules/no-irregular-whitespace
    'no-irregular-whitespace': [ERROR, {
      skipStrings: true,
      skipComments: true,
      skipRegExps: true,
      skipTemplates: true
    }],

    // disallow Number Literals That Lose Precision
    // https://eslint.org/docs/rules/no-loss-of-precision
    'no-loss-of-precision': ERROR,

    // disallow characters which are made with multiple code points in character class syntax
    // https://eslint.org/docs/rules/no-misleading-character-class
    'no-misleading-character-class': ERROR,

    // disallow the use of object properties of the global object (Math and JSON) as functions
    // https://eslint.org/docs/rules/no-obj-calls
    'no-obj-calls': ERROR,

    // disallow returning values from Promise executor functions
    // https://eslint.org/docs/rules/no-promise-executor-return
    'no-promise-executor-return': ERROR,

    // disallow use of Object.prototypes builtins directly
    // https://eslint.org/docs/rules/no-prototype-builtins
    'no-prototype-builtins': ERROR,

    // disallow multiple spaces in a regular expression literal
    // https://eslint.org/docs/rules/no-regex-spaces
    'no-regex-spaces': ERROR,

    // disallow returning values from setters
    // https://eslint.org/docs/rules/no-setter-return
    'no-setter-return': ERROR,

    // disallow sparse arrays
    // https://eslint.org/docs/rules/no-sparse-arrays
    'no-sparse-arrays': ERROR,

    // disallow template literal placeholder syntax in regular strings
    // https://eslint.org/docs/rules/no-template-curly-in-string
    'no-template-curly-in-string': ERROR,

    // avoid code that looks like two expressions but is actually one
    // https://eslint.org/docs/rules/no-unexpected-multiline
    'no-unexpected-multiline': ERROR,

    // disallow unreachable statements after a return, throw, continue, or break statement
    // https://eslint.org/docs/rules/no-unreachable
    'no-unreachable': ERROR,

    // disallow loops with a body that allows only one iteration
    // https://eslint.org/docs/rules/no-unreachable-loop
    'no-unreachable-loop': [ERROR, {
      ignore: [
        // WhileStatement
        // DoWhileStatement
        // ForStatement
        // ForInStatement,
        // ForOfStatement
      ]
    }],

    // disallow return/throw/break/continue inside finally blocks
    // https://eslint.org/docs/rules/no-unsafe-finally
    'no-unsafe-finally': ERROR,

    // disallow negating the left operand of relational operators
    // https://eslint.org/docs/rules/no-unsafe-negation
    'no-unsafe-negation': [ERROR, {
      enforceForOrderingRelations: true
    }],

    // disallow use of optional chaining in contexts where the undefined value is not allowed
    // https://eslint.org/docs/rules/no-unsafe-optional-chaining
    'no-unsafe-optional-chaining': [ERROR, {
      disallowArithmeticOperators: true
    }],

    // disallow unused private class members
    // https://eslint.org/docs/rules/no-unused-private-class-members
    'no-unused-private-class-members': ERROR,

    // disallow useless backreferences in regular expressions
    // https://eslint.org/docs/rules/no-useless-backreference
    'no-useless-backreference': ERROR,

    // DEPRECATED in favor of no-unsafe-negation
    // https://eslint.org/docs/rules/no-negated-in-lhs
    'no-negated-in-lhs': OFF,

    // disallow assignments that can lead to race conditions due to usage of await or yield
    // https://eslint.org/docs/rules/require-atomic-updates
    'require-atomic-updates': [ERROR, {
      allowProperties: false
    }],

    // disallow comparisons with the value NaN
    // https://eslint.org/docs/rules/use-isnan
    'use-isnan': [ERROR, {
      enforceForSwitchCase: true,
      enforceForIndexOf: true
    }],

    // DEPRECATED as of ESLint v5.10.0.
    // ensure JSDoc comments are valid
    // https://eslint.org/docs/rules/valid-jsdoc
    'valid-jsdoc': OFF,

    // ensure that the results of typeof are compared against a valid string
    // https://eslint.org/docs/rules/valid-typeof
    'valid-typeof': [ERROR, {
      requireStringLiterals: true
    }]
  }
};
