const OFF   = 0;    // OFF   or 0 - turn the rule off
const WARN  = 1;    // WARN  or 1 - turn the rule on as a warning (does not affect exit code)
const ERROR = 2;    // ERROR or 2 - turn the rule on as an error (exit code will be 1)


module.exports = {
  env: {
    es6: true
  },
  parserOptions: {
    ecmaVersion: '2015',
    sourceType: 'module'
  },

  rules: {
    // enforce no braces where they can be omitted
    // https://eslint.org/docs/rules/arrow-body-style
    'arrow-body-style': [ERROR, 'as-needed', {
      requireReturnForObjectLiteral: false
    }],

    // require parens in arrow function arguments
    // https://eslint.org/docs/rules/arrow-parens
    'arrow-parens': [ERROR, 'as-needed', {
      requireForBlockBody: false
    }],

    // require space before/after arrow function's arrow
    // https://eslint.org/docs/rules/arrow-spacing
    'arrow-spacing': [ERROR, {
      before: true,
      after: true
    }],

    // verify super() callings in constructors
    // https://eslint.org/docs/rules/constructor-super
    'constructor-super': ERROR,

    // enforce the spacing around the * in generator functions
    // https://eslint.org/docs/rules/generator-star-spacing
    'generator-star-spacing': [ERROR, {
      before: false,
      after: true
    }],

    // disallow modifying variables of class declarations
    // https://eslint.org/docs/rules/no-class-assign
    'no-class-assign': ERROR,

    // disallow arrow functions where they could be confused with comparisons
    // https://eslint.org/docs/rules/no-confusing-arrow
    'no-confusing-arrow': [ERROR, {
      allowParens: true
    }],

    // disallow modifying variables that are declared using const
    // https://eslint.org/docs/rules/no-const-assign
    'no-const-assign': ERROR,

    // disallow duplicate class members
    // https://eslint.org/docs/rules/no-dupe-class-members
    'no-dupe-class-members': ERROR,

    // replaced by import/no-duplicates
    // https://eslint.org/docs/rules/no-duplicate-imports
    'no-duplicate-imports': OFF,

    // disallow symbol constructor
    // https://eslint.org/docs/rules/no-new-symbol
    'no-new-symbol': ERROR,

    // disallow specified names in exports
    // https://eslint.org/docs/rules/no-restricted-exports
    'no-restricted-exports': [ERROR, {
      restrictedNamedExports: [
        'default', // use `module.exports =` to provide a default export
        'then'     // this will cause tons of confusion when your module is dynamically `import()`ed
      ]
    }],

    // disallow specific imports
    // https://eslint.org/docs/rules/no-restricted-imports
    'no-restricted-imports': [OFF, {
      paths: [],
      patterns: []
    }],

    // disallow to use this/super before super() calling in constructors.
    // https://eslint.org/docs/rules/no-this-before-super
    'no-this-before-super': ERROR,

    // disallow useless computed property keys
    // https://eslint.org/docs/rules/no-useless-computed-key
    'no-useless-computed-key': [ERROR, {
      enforceForClassMembers: true
    }],

    // disallow unnecessary constructor
    // https://eslint.org/docs/rules/no-useless-constructor
    'no-useless-constructor': ERROR,

    // disallow renaming import, export, and destructured assignments to the same name
    // https://eslint.org/docs/rules/no-useless-rename
    'no-useless-rename': [ERROR, {
      ignoreDestructuring: false,
      ignoreExport: false,
      ignoreImport: false
    }],

    // require let or const instead of var
    // https://eslint.org/docs/rules/no-var
    'no-var': ERROR,

    // require method and property shorthand syntax for object literals
    // https://eslint.org/docs/rules/object-shorthand
    'object-shorthand': [ERROR, 'always', {
      avoidExplicitReturnArrows: true,
      avoidQuotes: true,
      ignoreConstructors: false
    }],

    // prefer using arrow functions as callbacks without changing the result
    // https://eslint.org/docs/rules/prefer-arrow-callback
    'prefer-arrow-callback': [ERROR, {
      allowNamedFunctions: false,
      allowUnboundThis: true
    }],

    // prefer using of const declaration for variables that are never modified after declared
    // https://eslint.org/docs/rules/prefer-const
    'prefer-const': [ERROR, {
      destructuring: 'any',
      ignoreReadBeforeAssign: true
    }],

    // prefer destructuring from arrays and objects
    // https://eslint.org/docs/rules/prefer-destructuring
    'prefer-destructuring': [ERROR, {
      VariableDeclarator: {
        array: false,
        object: true
      },
      AssignmentExpression: {
        array: true,
        object: false
      }
    }, {
      enforceForRenamedProperties: false
    }],

    // prefer binary, octal, and hexadecimal literals instead of parseInt()
    // https://eslint.org/docs/rules/prefer-numeric-literals
    'prefer-numeric-literals': ERROR,

    // DEPRECATED as of ESLint v3.9.0
    // suggest using Reflect methods where applicable
    // https://eslint.org/docs/rules/prefer-reflect
    'prefer-reflect': OFF,

    // prefer rest parameters instead of arguments
    // https://eslint.org/docs/rules/prefer-rest-params
    'prefer-rest-params': ERROR,

    // prefer using the spread operator instead of .apply()
    // https://eslint.org/docs/rules/prefer-spread
    'prefer-spread': ERROR,

    // prefer using template literals instead of string concatenation
    // https://eslint.org/docs/rules/prefer-template
    'prefer-template': ERROR,

    // disallow generator functions that do not have yield
    // https://eslint.org/docs/rules/require-yield
    'require-yield': ERROR,

    // enforce spacing between object rest-spread
    // https://eslint.org/docs/rules/rest-spread-spacing
    'rest-spread-spacing': [ERROR, 'never'],

    // enforce import sorting
    // https://eslint.org/docs/rules/sort-imports
    'sort-imports': [OFF, {
      allowSeparatedGroups: true,
      ignoreCase: false,
      ignoreDeclarationSort: false,
      ignoreMemberSort: false,
      memberSyntaxSortOrder: ['none', 'all', 'multiple', 'single']
    }],

    // require a Symbol description
    // https://eslint.org/docs/rules/symbol-description
    'symbol-description': ERROR,

    // enforce usage of spacing in template strings
    // https://eslint.org/docs/rules/template-curly-spacing
    'template-curly-spacing': [ERROR, 'never'],

    // enforce spacing around the * in yield* expressions
    // https://eslint.org/docs/rules/yield-star-spacing
    'yield-star-spacing': [ERROR, 'after']
  }
};
