const OFF   = 0;    // OFF   or 0 - turn the rule off
const WARN  = 1;    // WARN  or 1 - turn the rule on as a warning (does not affect exit code)
const ERROR = 2;    // ERROR or 2 - turn the rule on as an error (exit code will be 1)


module.exports = {
  env: {
    node: true
  },
  parserOptions: {
    ecmaVersion: '2022',
    sourceType: 'module'
  },
  plugins: [
    'n'
  ],
  rules: {
    /*
     * Best Practices
     */

    // prevent using deprecated APIs
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-deprecated-api.md
    'n/no-deprecated-api': [ERROR, {
      version: '>=18.0.0',
      ignoreModuleItems: [],
      ignoreGlobalItems: []
    }],


    /*
     * Possible Errors
     */

    // require error handling in callbacks
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/handle-callback-err.md
    'n/handle-callback-err': [ERROR, '^(err|error)$'],

    // ensure Node.js-style error-first callback pattern is followed
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-callback-literal.md
    'n/no-callback-literal': ERROR,

    // disallow the assignment to `exports`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-exports-assign.md
    'n/no-exports-assign': ERROR,

    // disallow `import` declarations which import extraneous modules
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-extraneous-import.md
    'n/no-extraneous-import': [ERROR, {
      allowModules: [],
      resolvePaths: []
    }],

    // disallow `require()` expressions which import extraneous modules
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-extraneous-require.md
    'n/no-extraneous-require': [ERROR, {
      allowModules: [],
      resolvePaths: [],
      tryExtensions: ['.js', '.json', '.node']
    }],

    // disallow `import` declarations which import non-existence modules
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-missing-import.md
    'n/no-missing-import': OFF,

    // disallow `require()` expressions which import non-existence modules
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-missing-require.md
    'n/no-missing-require': OFF,

    // disallow `new` operators with calls to `require`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-new-require.md
    'n/no-new-require': ERROR,

    // disallow string concatenation with `__dirname` and `__filename`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-path-concat.md
    'n/no-path-concat': ERROR,

    // disallow the use of `process.exit()`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-process-exit.md
    'n/no-process-exit': OFF,

    // disallow `bin` files that npm ignores
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-unpublished-bin.md
    'n/no-unpublished-bin': OFF,

    // disallow `import` declarations which import private modules
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-unpublished-import.md
    'n/no-unpublished-import': OFF,

    // disallow `require()` expressions which import private modules
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-unpublished-require.md
    'n/no-unpublished-require': OFF,

    // disallow unsupported ECMAScript built-ins on the specified version
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-unsupported-features/es-builtins.md
    'n/no-unsupported-features/es-builtins': [ERROR, {
      version: '>=18.0.0',
      ignores: []
    }],

    // disallow unsupported ECMAScript syntax on the specified version
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-unsupported-features/es-syntax.md
    'n/no-unsupported-features/es-syntax': [ERROR, {
      version: '>=18.0.0',
      ignores: []
    }],

    // disallow unsupported Node.js built-in APIs on the specified version
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-unsupported-features/node-builtins.md
    'n/no-unsupported-features/node-builtins': [ERROR, {
      version: '>=18.0.0',
      ignores: []
    }],

    // make `process.exit()` expressions the same code path as `throw`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/process-exit-as-throw.md
    'n/process-exit-as-throw': ERROR,

    // suggest correct usage of shebang
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/shebang.md
    'n/shebang': OFF,


    /*
     * Stylistic Issues
     */

    // require `return` statements after callbacks
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/callback-return.md
    'n/callback-return': ERROR,

    // enforce either `module.exports` or `exports`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/exports-style.md
    'n/exports-style': OFF,

    // enforce the style of file extensions in `import` declarations
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/file-extension-in-import.md
    'n/file-extension-in-import': OFF,

    // require `require()` calls to be placed at top-level module scope
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/global-require.md
    'n/global-require': ERROR,

    // disallow `require` calls to be mixed with regular variable declarations
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-mixed-requires.md
    'n/no-mixed-requires': [ERROR, {
      grouping: false,
      allowCall: false
    }],

    // disallow the use of `process.env`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-process-env.md
    'n/no-process-env': OFF,

    // disallow specified modules when loaded by `import` declarations
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-restricted-import.md
    'n/no-restricted-import': OFF,

    // disallow specified modules when loaded by `require`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-restricted-require.md
    'n/no-restricted-require': OFF,

    // disallow synchronous methods
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/no-sync.md
    'n/no-sync': OFF,

    // enforce either `Buffer` or `require('buffer').Buffer`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-global/buffer.md
    'n/prefer-global/buffer': [ERROR, 'always'],

    // enforce either `console` or `require('console')`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-global/console.md
    'n/prefer-global/console': [ERROR, 'always'],

    // enforce either `process` or `require('process')`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-global/process.md
    'n/prefer-global/process': [ERROR, 'always'],

    // enforce either `TextDecoder` or `require('util').TextDecoder`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-global/text-decoder.md
    'n/prefer-global/text-decoder': [ERROR, 'always'],

    // enforce either `TextEncoder` or `require('util').TextEncoder`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-global/text-encoder.md
    'n/prefer-global/text-encoder': [ERROR, 'always'],

    // enforce either `URLSearchParams` or `require('url').URLSearchParams`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-global/url-search-params.md
    'n/prefer-global/url-search-params': [ERROR, 'always'],

    // enforce either `URL` or `require('url').URL`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-global/url.md
    'n/prefer-global/url': [ERROR, 'always'],

    // enforce `require('dns').promises`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-promises/dns.md
    'n/prefer-promises/dns': ERROR,

    // enforce `require('fs').promises`
    // https://github.com/rhansen/eslint-plugin-n/blob/master/docs/rules/prefer-promises/fs.md
    'n/prefer-promises/fs': ERROR
  }
};
