const OFF   = 0;    // OFF   or 0 - turn the rule off
const WARN  = 1;    // WARN  or 1 - turn the rule on as a warning (does not affect exit code)
const ERROR = 2;    // ERROR or 2 - turn the rule on as an error (exit code will be 1)


module.exports = {
  parserOptions: {
    ecmaVersion: '2022',
    sourceType: 'module'
  },
  rules: {
    // enforce line breaks after opening and before closing array brackets
    // https://eslint.org/docs/rules/array-bracket-newline
    'array-bracket-newline': [ERROR, 'consistent'],

    // enforce line breaks between array elements
    // https://eslint.org/docs/rules/array-element-newline
    'array-element-newline': [ERROR, 'consistent'],

    // enforce spacing inside array brackets
    // https://eslint.org/docs/rules/array-bracket-spacing
    'array-bracket-spacing': [ERROR, 'never'],

    // enforce spacing inside single-line blocks
    // https://eslint.org/docs/rules/block-spacing
    'block-spacing': [ERROR, 'always'],

    // enforce one true brace style
    // https://eslint.org/docs/rules/brace-style
    'brace-style': [ERROR, '1tbs', {
      allowSingleLine: true
    }],

    // require camel case names
    // https://eslint.org/docs/rules/camelcase
    camelcase: [ERROR, {
      properties: 'never',
      ignoreDestructuring: false,
      ignoreImports: false,
      ignoreGlobals: true,
      allow: ['^UNSAFE_']
    }],

    // enforce or disallow capitalization of the first letter of a comment
    // https://eslint.org/docs/rules/capitalized-comments
    'capitalized-comments': [OFF, 'never', {
      line: {
        ignorePattern: '.*',
        ignoreInlineComments: true,
        ignoreConsecutiveComments: true
      },
      block: {
        ignorePattern: '.*',
        ignoreInlineComments: true,
        ignoreConsecutiveComments: true
      }
    }],

    // require trailing commas in multiline object literals
    // https://eslint.org/docs/rules/comma-dangle
    'comma-dangle': [ERROR, {
      arrays: 'never',
      objects: 'never',
      imports: 'never',
      exports: 'never',
      functions: 'never'
    }],

    // enforce spacing before and after comma
    // https://eslint.org/docs/rules/comma-spacing
    'comma-spacing': [ERROR, {
      before: false,
      after: true
    }],

    // enforce one true comma style
    // https://eslint.org/docs/rules/comma-style
    'comma-style': [ERROR, 'last', {
      exceptions: {
        ArrayExpression: false,
        ArrayPattern: false,
        ArrowFunctionExpression: false,
        CallExpression: false,
        FunctionDeclaration: false,
        FunctionExpression: false,
        ImportDeclaration: false,
        ObjectExpression: false,
        ObjectPattern: false,
        VariableDeclaration: false,
        NewExpression: false
      }
    }],

    // disallow padding inside computed properties
    // https://eslint.org/docs/rules/computed-property-spacing
    'computed-property-spacing': [ERROR, 'never', {
      enforceForClassMembers: true
    }],

    // enforce consistent naming when capturing the current execution context
    // https://eslint.org/docs/rules/consistent-this
    'consistent-this': [ERROR, 'that'],

    // enforce newline at the end of file, with no multiple empty lines
    // https://eslint.org/docs/rules/eol-last
    'eol-last': [ERROR, 'always'],

    // enforce line breaks between arguments of a function call
    // https://eslint.org/docs/rules/function-call-argument-newline
    'function-call-argument-newline': [OFF, 'consistent'],

    // enforce spacing between functions and their invocations
    // https://eslint.org/docs/rules/func-call-spacing
    'func-call-spacing': [ERROR, 'never'],

    // require function names to match the name of the variable or property to which they are assigned
    // https://eslint.org/docs/rules/func-name-matching
    'func-name-matching': [WARN, 'always', {
      considerPropertyDescriptor: true,
      includeCommonJSModuleExports: false
    }],

    // require function expressions to have a name
    // https://eslint.org/docs/rules/func-names
    'func-names': [ERROR, 'as-needed', {
      generators: 'as-needed'
    }],

    // enforce use of function declarations or expressions
    // https://eslint.org/docs/rules/func-style
    'func-style': [ERROR, 'declaration', {
      allowArrowFunctions: true
    }],

    // enforce consistent line breaks inside function parentheses
    // https://eslint.org/docs/rules/function-paren-newline
    'function-paren-newline': [ERROR, 'consistent'],

    // deprecated in favor of 'id-denyList'
    // https://eslint.org/docs/rules/id-blacklist
    'id-blacklist': [OFF, 'data', 'err', 'e', 'cb', 'callback'],

    // disallow specified identifiers from being used
    // https://eslint.org/docs/rules/id-denylist
    'id-denylist': [ERROR, 'data', 'err', 'e', 'cb', 'callback'],

    // enforce minimum and maximum identifier lengths
    // https://eslint.org/docs/rules/id-length
    'id-length': [ERROR, {
      min: 2,
      max: 32,
      properties: 'always',
      exceptions: ['i', 'j', 'k', 't', 'x', 'y', 'z']
    }],

    // require identifiers to match the provided regular expression
    // https://eslint.org/docs/rules/id-match
    'id-match': OFF,

    // enforce the location of arrow function bodies with implicit returns
    // https://eslint.org/docs/rules/implicit-arrow-linebreak
    'implicit-arrow-linebreak': [ERROR, 'beside'],

    // this option sets a specific tab width for your code
    // https://eslint.org/docs/rules/indent
    indent: [ERROR, 2, {
      SwitchCase: 1,
      VariableDeclarator: 1,
      outerIIFEBody: 1,
      MemberExpression: 1,
      FunctionDeclaration: {
        parameters: 1,
        body: 1
      },
      FunctionExpression: {
        parameters: 1,
        body: 1
      },
      CallExpression: {
        arguments: 1
      },
      ArrayExpression: 1,
      ObjectExpression: 1,
      ImportDeclaration: 1,
      ignoreComments: false,
      flatTernaryExpressions: false,
      offsetTernaryExpressions: true,
      // list derived from https://github.com/benjamn/ast-types/blob/HEAD/def/jsx.js
      ignoredNodes: [
        'TemplateLiteral *',
        'JSXElement',
        'JSXElement > *',
        'JSXAttribute',
        'JSXIdentifier',
        'JSXNamespacedName',
        'JSXMemberExpression',
        'JSXSpreadAttribute',
        'JSXExpressionContainer',
        'JSXOpeningElement',
        'JSXClosingElement',
        'JSXFragment',
        'JSXOpeningFragment',
        'JSXClosingFragment',
        'JSXText',
        'JSXEmptyExpression',
        'JSXSpreadChild'
      ]
    }],

    // prefer double quotes in JSX attributes
    // https://eslint.org/docs/rules/jsx-quotes
    'jsx-quotes': [OFF, 'prefer-double'],

    // enforce spacing between keys and values in object literal properties
    // https://eslint.org/docs/rules/key-spacing
    'key-spacing': [ERROR, {
      beforeColon: false,
      afterColon: true,
      mode: 'strict'
    }],

    // require a space before & after certain keywords
    // https://eslint.org/docs/rules/keyword-spacing
    'keyword-spacing': [ERROR, {
      before: true,
      after: true,
      overrides: {
        return: { after: true },
        throw: { after: true },
        case: { after: true }
      }
    }],

    // enforce position of line comments
    // https://eslint.org/docs/rules/line-comment-position
    'line-comment-position': [OFF, {
      position: 'above',
      ignorePattern: '',
      applyDefaultIgnorePatterns: true
    }],

    // disallow mixed 'LF' and 'CRLF' as linebreaks
    // https://eslint.org/docs/rules/linebreak-style
    'linebreak-style': [ERROR, 'unix'],

    // require an empty line between class members
    // https://eslint.org/docs/rules/lines-between-class-members
    'lines-between-class-members': [ERROR, 'always', {
      exceptAfterSingleLine: false
    }],

    // enforce empty lines around comments
    // https://eslint.org/docs/rules/lines-around-comment
    'lines-around-comment': [OFF, {
      beforeBlockComment: true,
      afterBlockComment: true,

      beforeLineComment: true,
      afterLineComment: false,

      allowBlockStart: true,
      allowBlockEnd: true,

      allowObjectStart: true,
      allowObjectEnd: false,

      allowArrayStart: false,
      allowArrayEnd: false,

      allowClassStart: true,
      allowClassEnd: false,

      applyDefaultIgnorePatterns: true,
      ignorePattern: ''
    }],

    // enforce maximum depth that blocks can be nested
    // https://eslint.org/docs/rules/max-depth
    'max-depth': [OFF, {
      max: 4
    }],

    // enforce maximum length of a line in your program
    // https://eslint.org/docs/rules/max-len
    'max-len': [ERROR, {
      code: 128,
      tabWidth: 2,
      comments: 128,
      ignorePattern: '',
      ignoreComments: false,
      ignoreTrailingComments: false,
      ignoreUrls: true,
      ignoreStrings: true,
      ignoreTemplateLiterals: true,
      ignoreRegExpLiterals: true
    }],

    // enforce maximum number of lines in a file
    // https://eslint.org/docs/rules/max-lines
    'max-lines': [ERROR, {
      max: 512,
      skipBlankLines: true,
      skipComments: true
    }],

    // enforce a maximum function length
    // https://eslint.org/docs/rules/max-lines-per-function
    'max-lines-per-function': [ERROR, {
      max: 64,
      skipBlankLines: true,
      skipComments: true,
      IIFEs: true
    }],

    // enforce maximum depth callbacks can be nested
    // https://eslint.org/docs/rules/max-nested-callbacks
    'max-nested-callbacks': [ERROR, {
      max: 2
    }],

    // enforce number of parameters that can be used in the function declaration
    // https://eslint.org/docs/rules/max-params
    'max-params': [ERROR, {
      max: 4
    }],

    // enforce maximum number of statement allowed in a function
    // https://eslint.org/docs/rules/max-statements
    'max-statements': [ERROR, {
      max: 8
    }],

    // enforce number of statements per line
    // https://eslint.org/docs/rules/max-statements-per-line
    'max-statements-per-line': [ERROR, {
      max: 1
    }],

    // enforce a particular style for multiline comments
    // https://eslint.org/docs/rules/multiline-comment-style
    'multiline-comment-style': [OFF, 'starred-block'],

    // enforce multiline ternary
    // https://eslint.org/docs/rules/multiline-ternary
    'multiline-ternary': [ERROR, 'always-multiline'],

    // enforce capital letter for constructors
    // https://eslint.org/docs/rules/new-cap
    'new-cap': [ERROR, {
      newIsCap: true,
      capIsNew: false,
      newIsCapExceptions: [],
      newIsCapExceptionPattern: '',
      capIsNewExceptions: ['Immutable.Map', 'Immutable.Set', 'Immutable.List'],
      capIsNewExceptionPattern: '',
      properties: true
    }],

    // disallow the omission of parentheses when invoking a constructor with no arguments
    // https://eslint.org/docs/rules/new-parens
    'new-parens': [ERROR, 'always'],

    // disallow an empty newline after var statement
    // https://eslint.org/docs/rules/newline-after-var
    'newline-after-var': OFF,

    // require an empty line before return statements
    // https://eslint.org/docs/rules/newline-before-return
    'newline-before-return': OFF,

    // enforces newline after each method call in the chain to make it more readable and easy to maintain
    // https://eslint.org/docs/rules/newline-per-chained-call
    'newline-per-chained-call': [ERROR, {
      ignoreChainWithDepth: 4
    }],

    // disallow use of the Array constructor
    // https://eslint.org/docs/rules/no-array-constructor
    'no-array-constructor': ERROR,

    // disallow use of bitwise operators
    // https://eslint.org/docs/rules/no-bitwise
    'no-bitwise': ERROR,

    // disallow use of the continue statement
    // https://eslint.org/docs/rules/no-continue
    'no-continue': OFF,

    // disallow comments inline after code
    // https://eslint.org/docs/rules/no-inline-comments
    'no-inline-comments': OFF,

    // disallow 'if' as the only statement in an 'else' block
    // https://eslint.org/docs/rules/no-lonely-if
    'no-lonely-if': ERROR,

    // disallow un-paren'd mixes of different operators
    // https://eslint.org/docs/rules/no-mixed-operators
    'no-mixed-operators': [ERROR, {
      groups: [
        ['+', '-', '*', '/', '%', '**'],                  // Arithmetic Operators
        ['&', '|', '^', '~', '<<', '>>', '>>>'],          // Bitwise Operators
        ['==', '!=', '===', '!==', '>', '>=', '<', '<='], // Comparison Operators
        ['&&', '||'],                                     // Logical Operators
        // ['??'],                                           // Coalesce Operator
        ['in', 'instanceof']                              // Relational Operators
        // ['?:']                                            // Ternary Operator
      ],
      allowSamePrecedence: false
    }],

    // disallow mixed spaces and tabs for indentation
    // https://eslint.org/docs/rules/no-mixed-spaces-and-tabs
    'no-mixed-spaces-and-tabs': ERROR,

    // disallow use of chained assignment expressions
    // https://eslint.org/docs/rules/no-multi-assign
    'no-multi-assign': ERROR,

    // disallow multiple empty lines and only one newline at the end
    // https://eslint.org/docs/rules/no-multiple-empty-lines
    'no-multiple-empty-lines': [ERROR, {
      max: 2,
      maxBOF: 0,
      maxEOF: 0
    }],

    // disallow negated conditions
    // https://eslint.org/docs/rules/no-negated-condition
    'no-negated-condition': OFF,

    // disallow nested ternary expressions
    // https://eslint.org/docs/rules/no-nested-ternary
    'no-nested-ternary': ERROR,

    // disallow use of the Object constructor
    // https://eslint.org/docs/rules/no-new-object
    'no-new-object': ERROR,

    // disallow use of unary operators, ++ and --
    // https://eslint.org/docs/rules/no-plusplus
    'no-plusplus': [ERROR, {
      allowForLoopAfterthoughts: true
    }],

    // disallow certain syntax forms
    // https://eslint.org/docs/rules/no-restricted-syntax
    'no-restricted-syntax': [
      2,
      {
        selector: 'ForInStatement',
        message: 'for..in loops iterate over the entire prototype chain, which is virtually never what you want. Use Object.{keys,values,entries}, and iterate over the resulting array.'
      },
      {
        selector: 'LabeledStatement',
        message: 'Labels are a form of GOTO; using them makes code confusing and hard to maintain and understand.'
      },
      {
        selector: 'WithStatement',
        message: '`with` is disallowed in strict mode because it makes code impossible to predict and optimize.'
      }
    ],

    // disallow space between function identifier and application
    // https://eslint.org/docs/rules/no-spaced-func
    'no-spaced-func': ERROR,

    // disallow tab characters entirely
    // https://eslint.org/docs/rules/no-tabs
    'no-tabs': ERROR,

    // disallow the use of ternary operators
    // https://eslint.org/docs/rules/no-ternary
    'no-ternary': OFF,

    // disallow trailing whitespace at the end of lines
    // https://eslint.org/docs/rules/no-trailing-spaces
    'no-trailing-spaces': [ERROR, {
      skipBlankLines: false,
      ignoreComments: false
    }],

    // disallow dangling underscores in identifiers
    // https://eslint.org/docs/rules/no-underscore-dangle
    'no-underscore-dangle': [ERROR, {
      allow: [],
      allowAfterThis: false,
      allowAfterSuper: false,
      enforceInMethodNames: true,
      allowFunctionParams: false
    }],

    // disallow the use of Boolean literals in conditional expressions
    // https://eslint.org/docs/rules/no-unneeded-ternary
    'no-unneeded-ternary': [ERROR, {
      defaultAssignment: false
    }],

    // disallow whitespace before properties
    // https://eslint.org/docs/rules/no-whitespace-before-property
    'no-whitespace-before-property': ERROR,

    // enforce the location of single-line statements
    // https://eslint.org/docs/rules/nonblock-statement-body-position
    'nonblock-statement-body-position': [OFF, 'beside', {
      overrides: {}
    }],

    // require padding inside curly braces
    // https://eslint.org/docs/rules/object-curly-spacing
    'object-curly-spacing': [ERROR, 'always', {
      arraysInObjects: true,
      objectsInObjects: true
    }],

    // enforce line breaks between braces
    // https://eslint.org/docs/rules/object-curly-newline
    'object-curly-newline': [ERROR, {
      ObjectExpression: {
        minProperties: 4,
        multiline: true,
        consistent: true
      },
      ObjectPattern: {
        minProperties: 4,
        multiline: false,
        consistent: true
      },
      ImportDeclaration: {
        minProperties: 4,
        multiline: true,
        consistent: true
      },
      ExportDeclaration: {
        minProperties: 4,
        multiline: true,
        consistent: true
      }
    }],

    // enforce 'same line' or 'multiple line' on object properties.
    // https://eslint.org/docs/rules/object-property-newline
    'object-property-newline': [ERROR, {
      allowAllPropertiesOnSameLine: true
    }],

    // disallow one var statement per function
    // https://eslint.org/docs/rules/one-var
    'one-var': [ERROR, {
      // var: 'never',
      // let: 'never',
      // const: 'never',
      // separateRequires: true,
      initialized: 'never',
      uninitialized: 'never'
    }],

    // require a newline around variable declaration
    // https://eslint.org/docs/rules/one-var-declaration-per-line
    'one-var-declaration-per-line': [ERROR, 'always'],

    // require assignment operator shorthand where possible or prohibit it entirely
    // https://eslint.org/docs/rules/operator-assignment
    'operator-assignment': [WARN, 'always'],

    // require operator at the beginning of the line in multiline statements
    // https://eslint.org/docs/rules/operator-linebreak
    'operator-linebreak': [ERROR, 'after', {
      overrides: {
        '=': 'none',
        '?': 'ignore',
        ':': 'ignore',
        '|>': 'ignore'
      }
    }],

    // disallow empty lines at the beginning and ending of block statements and classe
    // https://eslint.org/docs/rules/padded-blocks
    'padded-blocks': [ERROR, {
      blocks: 'never',
      classes: 'never',
      switches: 'never'
    }, {
      allowSingleLineBlocks: true
    }],

    // disallow padding lines between statements
    // https://eslint.org/docs/rules/padding-line-between-statements
    'padding-line-between-statements': [ERROR, {
      blankLine: 'any',
      prev: '*',
      next: 'return'
    }, {
      blankLine: 'any',
      prev: ['const', 'let', 'var'],
      next: '*'
    }, {
      blankLine: 'any',
      prev: ['const', 'let', 'var'],
      next: ['const', 'let', 'var']
    }],

    // disallow the use of Math.pow in favor of the ** operator
    // https://eslint.org/docs/rules/prefer-exponentiation-operator
    'prefer-exponentiation-operator': ERROR,

    // prefer use of an object spread over Object.assign
    // https://eslint.org/docs/rules/prefer-object-spread
    'prefer-object-spread': ERROR,

    // require quotes around object literal property names
    // https://eslint.org/docs/rules/quote-props.html
    'quote-props': [ERROR, 'as-needed', {
      keywords: false,
      unnecessary: true,
      numbers: false
    }],

    // specify whether double or single quotes should be used
    // https://eslint.org/docs/rules/quotes
    quotes: [ERROR, 'single', {
      avoidEscape: true,
      allowTemplateLiterals: true
    }],

    // deprecated as of ESLint v5.10.0
    // https://eslint.org/docs/rules/require-jsdoc
    'require-jsdoc': OFF,

    // require or disallow use of semicolons instead of ASI
    // https://eslint.org/docs/rules/semi
    semi: [ERROR, 'always'],

    // enforce spacing before and after semicolons
    // https://eslint.org/docs/rules/semi-spacing
    'semi-spacing': [ERROR, {
      before: false,
      after: true
    }],

    // enforce location of semicolons
    // https://eslint.org/docs/rules/semi-style
    'semi-style': [ERROR, 'last'],

    // requires object keys to be sorted
    // https://eslint.org/docs/rules/sort-keys
    'sort-keys': [OFF, 'asc', {
      caseSensitive: false,
      natural: true
    }],

    // sort variables within the same declaration block
    // https://eslint.org/docs/rules/sort-vars
    'sort-vars': [OFF, {
      ignoreCase: false
    }],

    // require or disallow space before blocks
    // https://eslint.org/docs/rules/space-before-blocks
    'space-before-blocks': [ERROR, {
      functions: 'always',
      keywords: 'always',
      classes: 'always'
    }],

    // disallow space before function opening parenthesis
    // https://eslint.org/docs/rules/space-before-function-paren
    'space-before-function-paren': [ERROR, {
      anonymous: 'never',
      named: 'never',
      asyncArrow: 'always'
    }],

    // require or disallow spaces inside parentheses
    // https://eslint.org/docs/rules/space-in-parens
    'space-in-parens': [ERROR, 'never'],

    // require spaces around operators
    // https://eslint.org/docs/rules/space-infix-ops
    'space-infix-ops': OFF,

    // disallow spaces before/after unary operators
    // https://eslint.org/docs/rules/space-unary-ops
    'space-unary-ops': [ERROR, {
      words: true,
      nonwords: false,
      overrides: {}
    }],

    // require a space immediately following the // or /* in a comment
    // https://eslint.org/docs/rules/spaced-comment
    'spaced-comment': [ERROR, 'always', {
      line: {
        markers: ['=', '!', '/'], // space here to support sprockets directives
        exceptions: ['-', '+']
      },
      block: {
        markers: ['=', '!'], // space here to support sprockets directives
        balanced: true,
        exceptions: ['-', '+']
      }
    }],

    // enforce spacing around colons of switch statements
    // https://eslint.org/docs/rules/switch-colon-spacing
    'switch-colon-spacing': [ERROR, {
      after: true,
      before: false
    }],

    // disallow spacing between template tags and their literals
    // https://eslint.org/docs/rules/template-tag-spacing
    'template-tag-spacing': [ERROR, 'never'],

    // disallow the Unicode Byte Order Mark
    // https://eslint.org/docs/rules/unicode-bom
    'unicode-bom': [ERROR, 'never'],

    // require regex literals to be wrapped in parentheses
    // https://eslint.org/docs/rules/wrap-regex
    'wrap-regex': OFF
  }
};
